package org.ks1.j3dbvh;

import java.util.*;
import javax.media.j3d.*;
import javax.vecmath.*;
import java.awt.*;
import java.awt.event.*;

public class SimpleMouseNavigationBehavior extends Behavior {
    TransformGroup tg;
    Transform3D t;

    Quat4d q = new Quat4d(0.0,0.0,0.0,1.0);
    Vector3d v = new Vector3d(0.0,0.0,0.0);
    double s = 1.0;

    public SimpleMouseNavigationBehavior(TransformGroup tg,Transform3D t) {
        this.tg = tg;
        this.t = t;
        s = t.get(q,v);
    }

    public void initialize() {
        t.set(q,v,s);
        tg.setTransform(t);
        WakeupOnAWTEvent w = new WakeupOnAWTEvent(MouseEvent.MOUSE_PRESSED);
        wakeupOn(w);
    }

    public void processStimulus(Enumeration criteria) {
        while (criteria.hasMoreElements()) {
            WakeupOnAWTEvent w = (WakeupOnAWTEvent)criteria.nextElement();
            AWTEvent e[] = w.getAWTEvent();
            for (int i=0;i<e.length;i++) {
                if (e[i].getID()==MouseEvent.MOUSE_PRESSED)
                    processMousePressed((MouseEvent)e[i]);
                else if (e[i].getID()==MouseEvent.MOUSE_DRAGGED)
                    processMouseDragged((MouseEvent)e[i]);
                else if (e[i].getID()==MouseEvent.MOUSE_RELEASED)
                    processMouseReleased((MouseEvent)e[i]);
            }
        }
    }

    int pressedX;
    int pressedY;
    int mouseX;
    int mouseY;
    long pressedTime;
    int mode;
    static final int ROTATION_MODE = 0;
    static final int TRANSLATION_MODE = 1;
    static final int SCALING_MODE = 2;
    void processMousePressed(MouseEvent me) {
        pressedX = me.getX();
        pressedY = me.getY();
        mouseX = me.getX();
        mouseY = me.getY();
        pressedTime = System.currentTimeMillis();
        if (me.getButton() == MouseEvent.BUTTON1)
            mode = ROTATION_MODE;
        else if (me.getButton() == MouseEvent.BUTTON2)
            mode = TRANSLATION_MODE;
        else if (me.getButton() == MouseEvent.BUTTON3)
            mode = SCALING_MODE;

        WakeupOnAWTEvent ws[] = new WakeupOnAWTEvent[2];
        ws[0] = new WakeupOnAWTEvent(MouseEvent.MOUSE_DRAGGED);
        ws[1] = new WakeupOnAWTEvent(MouseEvent.MOUSE_RELEASED);
        WakeupOr w = new WakeupOr(ws);
        wakeupOn(w);
    }

    void processMouseDragged(MouseEvent me) {
        double dX = (double)(me.getX() - mouseX);
        double dY = (double)(me.getY() - mouseY);
        if (mode == ROTATION_MODE) {
            double cos = Math.cos((dX*Math.PI/180.0)/2.0);
            double sin = Math.sin((dX*Math.PI/180.0)/2.0);
            Quat4d qTmp = new Quat4d(0.0,sin,0.0,cos);
            qTmp.mul(q);
            q = qTmp;
            cos = Math.cos((dY*Math.PI/180.0)/2.0);
            sin = Math.sin((dY*Math.PI/180.0)/2.0);
            qTmp = new Quat4d(sin,0.0,0.0,cos);
            qTmp.mul(q);
            q = qTmp;
        } else if (mode == TRANSLATION_MODE) {
            double tx = 0.0;
            double ty = 0.0;
            double tz = 0.0;
            if ((me.getModifiers() & MouseEvent.SHIFT_MASK)!=0) {
                tx =  dX*s/100.0;
                tz =  dY*s/100.0;
            } else {
                tx =  dX*s/100.0;
                ty = -dY*s/100.0;
            }
            v.add(new Vector3d(tx,ty,tz));
        } else if (mode == SCALING_MODE) {
            s = s - s*dY/100.0;
        }
        mouseX = me.getX();
        mouseY = me.getY();

        t.set(q,v,s);
        tg.setTransform(t);

        WakeupOnAWTEvent ws[] = new WakeupOnAWTEvent[2];
        ws[0] = new WakeupOnAWTEvent(MouseEvent.MOUSE_DRAGGED);
        ws[1] = new WakeupOnAWTEvent(MouseEvent.MOUSE_RELEASED);
        WakeupOr w = new WakeupOr(ws);
        wakeupOn(w);
    }
    void processMouseReleased(MouseEvent me) {
        WakeupOnAWTEvent w = new WakeupOnAWTEvent(MouseEvent.MOUSE_PRESSED);
        wakeupOn(w);
    }
}
