package org.ks1.j3dbvh;

import javax.media.j3d.*;
import javax.vecmath.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import java.io.*;
import java.net.*;

public class Viewer implements ActionListener {
    Frame frame;
    JTextField urlTextField;
    JButton refButton;
    JFileChooser fileChooser;
    Canvas3D canvas3D;
    JButton proxyButton;
    JLabel statusBarLabel;

    BranchGroup rootGroup;
    BranchGroup targetGroup;

    public Viewer(String args[]) {
        frame = new Frame("j3dbvh Viewer");
        Box baseBox = Box.createVerticalBox();
        frame.add(baseBox);

        Box box1 = Box.createHorizontalBox();
        baseBox.add(box1);
        box1.add(new JLabel("URL:"));
        urlTextField = new JTextField();
        try {
            File f = new File(args[0]);
            URL url = f.toURL();
            urlTextField.setText(url.toExternalForm());
        } catch(Exception e) {
            urlTextField.setText("file:///error.bvh");
        }
        urlTextField.addActionListener(this);
urlTextField.setMaximumSize(new Dimension(3000,30));
        box1.add(urlTextField);
        refButton = new JButton("ref");
        refButton.addActionListener(this);
        box1.add(refButton);

        Box box2 = Box.createHorizontalBox();
        baseBox.add(box2);

        canvas3D = makeCanvas3D();
        box2.add(canvas3D);

        Box controlBox = makeControlBox();
        box2.add(controlBox);

        statusBarLabel = new JLabel("Status:");
        baseBox.add(statusBarLabel);

        frame.addWindowListener(new WindowAdapter() {
            public void windowClosing(WindowEvent we) {
                frame.dispose();
                System.exit(0);
            }
        });
        frame.pack();
        frame.setVisible(true);

        loadBVH();
    }

    Canvas3D makeCanvas3D() {
        GraphicsEnvironment ge = GraphicsEnvironment.getLocalGraphicsEnvironment();
        GraphicsDevice gd = ge.getDefaultScreenDevice();
        GraphicsConfiguration gcs[] = gd.getConfigurations();
        GraphicsConfigTemplate3D gct3d = new GraphicsConfigTemplate3D();
        GraphicsConfiguration gc = gct3d.getBestConfiguration(gcs);
        Canvas3D canvas3D = new Canvas3D(gc);
        canvas3D.setSize(300,300);

        PhysicalBody physicalBody = new PhysicalBody();
        PhysicalEnvironment physicalEnvironment = new PhysicalEnvironment();

        BranchGroup branchGroupForViewPlatform = new BranchGroup();
        Transform3D transform3DForViewPlatform = new Transform3D();
        transform3DForViewPlatform.set(new Vector3f(0.0f,0.0f,2.0f));
        ViewPlatform viewPlatform = new ViewPlatform();
        TransformGroup transformGroupForViewPlatform = new TransformGroup(transform3DForViewPlatform);
        transformGroupForViewPlatform.addChild(viewPlatform);
        branchGroupForViewPlatform.addChild(transformGroupForViewPlatform);
//        PointLight light = new PointLight();
//        AmbientLight light = new AmbientLight();
        DirectionalLight light = new DirectionalLight();
        light.setInfluencingBounds(new BoundingSphere(new Point3d(0.0,0.0,0.0),100.0));
        branchGroupForViewPlatform.addChild(light);

        View view = new View();
        view.addCanvas3D(canvas3D);
        view.setPhysicalBody(physicalBody);
        view.setPhysicalEnvironment(physicalEnvironment);
        view.attachViewPlatform(viewPlatform);

        VirtualUniverse virtualUniverse = new VirtualUniverse();
        Locale locale = new Locale(virtualUniverse);

        BranchGroup topGroup = new BranchGroup();
        TransformGroup tg = new TransformGroup();
        tg.setCapability(TransformGroup.ALLOW_TRANSFORM_WRITE);
        topGroup.addChild(tg);
        Transform3D t = new Transform3D();
        tg.setTransform(t);
        SimpleMouseNavigationBehavior smnb =
            new SimpleMouseNavigationBehavior(tg,t);
        BoundingSphere bs = new BoundingSphere(new Point3d(0.0,0.0,0.0),100.0);
        smnb.setSchedulingBounds(bs);
        topGroup.addChild(smnb);
        rootGroup = new BranchGroup();
        rootGroup.setCapability(BranchGroup.ALLOW_CHILDREN_READ);
        rootGroup.setCapability(BranchGroup.ALLOW_CHILDREN_EXTEND);
        rootGroup.setCapability(BranchGroup.ALLOW_CHILDREN_WRITE);
        tg.addChild(rootGroup);

        locale.addBranchGraph(branchGroupForViewPlatform);
        locale.addBranchGraph(topGroup);

        return canvas3D;
    }

    Box makeControlBox() {
        Box controlBox = Box.createVerticalBox();

        proxyButton = new JButton("Proxy Configuration");
        proxyButton.addActionListener(this);
        controlBox.add(proxyButton);

        controlBox.add(Box.createVerticalGlue());

        return controlBox;
    }

    public void actionPerformed(ActionEvent ae) {
        if (ae.getSource() == urlTextField) {
            loadBVH();
        } else if (ae.getSource() == proxyButton) {
            proxyConfig();
        } else if (ae.getSource() == refButton) {
            ref();
        }
    }

    void loadBVH() {
        statusBarLabel.setText("Status: Loading ... ");
        statusBarLabel.repaint();
        String urlString = urlTextField.getText();
        BVHData bvh = BVHLoader.load(urlString);

        if (targetGroup != null) {
            targetGroup.detach();
        }
        targetGroup = new BranchGroup();
        targetGroup.setCapability(BranchGroup.ALLOW_DETACH);
        Node n = bvh.getNode();
        targetGroup.addChild(n);
        rootGroup.addChild(targetGroup);
        bvh.start();
        statusBarLabel.setText("Status: Loading ... done.");
    }

    ProxyPanel proxyPanel = null;
    String proxyData[] = {"","","",""};
    void proxyConfig() {
        if (proxyPanel == null) {
            proxyPanel = new ProxyPanel();
        }
        proxyPanel.setData(proxyData);

        Object options[] = {"Ok","Cancel"};
        int result = JOptionPane.showOptionDialog(frame,proxyPanel,"proxy",
                                                  JOptionPane.DEFAULT_OPTION,
                                                  JOptionPane.INFORMATION_MESSAGE,
                                                  null,options,options[0]);
        if (result==0) {
            proxyData = proxyPanel.getData();
            if (!proxyData[0].equals("")) {
                System.getProperties().put("proxySet","true");
                System.getProperties().put("proxyHost",proxyData[0]);
                System.getProperties().put("proxyPort",proxyData[1]);
            } else {
                System.getProperties().put("proxySet","false");
            }
            if (!proxyData[2].equals("")) {
                System.getProperties().put("ftpProxySet","true");
                System.getProperties().put("ftpProxyHost",proxyData[2]);
                System.getProperties().put("ftpProxyPort",proxyData[3]);
            } else {
                System.getProperties().put("ftpProxySet","false");
            }
        }
    }

    class ProxyPanel extends JPanel {
        JTextField httpHostTF;
        JTextField httpPortTF;
        JTextField ftpHostTF;
        JTextField ftpPortTF;

        public ProxyPanel() {
            Box b0 = Box.createVerticalBox();
            Box b1 = Box.createHorizontalBox();
            Box b2 = Box.createHorizontalBox();

            this.add(b0);
            b0.add(b1);
            b0.add(b2);

            b1.add(new JLabel("http: host"));
            httpHostTF = new JTextField(40);
            b1.add(httpHostTF);
            b1.add(new JLabel(" port "));
            httpPortTF = new JTextField(5);
            b1.add(httpPortTF);

            b2.add(new JLabel("ftp: host"));
            ftpHostTF = new JTextField(40);
            b2.add(ftpHostTF);
            b2.add(new JLabel(" port "));
            ftpPortTF = new JTextField(5);
            b2.add(ftpPortTF);
        }

        public void setData(String proxyData[]) {
            httpHostTF.setText(proxyData[0]);
            httpPortTF.setText(proxyData[1]);
            ftpHostTF.setText(proxyData[2]);
            ftpPortTF.setText(proxyData[3]);
        }

        public String[] getData() {
            String ret[] = new String[4];
            ret[0] = httpHostTF.getText();
            ret[1] = httpPortTF.getText();
            ret[2] = ftpHostTF.getText();
            ret[3] = ftpPortTF.getText();
            return ret;
        }
    }

    void ref() {
        if (fileChooser == null) {
            fileChooser = new JFileChooser(".");
        }
        int returnVal = fileChooser.showOpenDialog(frame);
        if(returnVal != JFileChooser.APPROVE_OPTION)
            return;

        File f = fileChooser.getSelectedFile();
        URL url = null;
        try{url = f.toURL();}catch(Exception e){;}
        if (url == null)
            return;
        urlTextField.setText(url.toExternalForm());
        loadBVH();
    }

    public static void main(String args[]) {
        System.setSecurityManager(null);
        final String argCopy[] = args;
        java.awt.EventQueue.invokeLater(new Runnable() {
            public void run() {
                new Viewer(argCopy);
            }
        });
    }
}
