package org.ks1.j3dbvh.bvh;

import java.io.*;
import java.util.*;

import org.ks1.j3dbvh.bvh.analysis.*;
import org.ks1.j3dbvh.bvh.node.*;
import org.ks1.j3dbvh.bvh.parser.*;
import org.ks1.j3dbvh.bvh.lexer.*;

import javax.media.j3d.*;
import javax.vecmath.*;

public class ActionGenerator extends DepthFirstAdapter {
    static BoundingSphere bs = new BoundingSphere(new Point3d(0.0,0.0,0.0),10.0);

    public static ActionGenerator generate(Reader reader) {
        try {
            Lexer lexer = new Lexer(new PushbackReader(reader));
            Parser parser = new Parser(lexer);

            org.ks1.j3dbvh.bvh.node.Node ast = parser.parse();

            ActionGenerator ag = new ActionGenerator();
            ast.apply(ag);

            return ag;
        } catch (Exception e) {
            e.printStackTrace();
        }
        return null;
    }

    public void start() {
        for (int i=0;i<behaviors.length;i++)
            behaviors[i].start();
    }

    public Group rootGroup = null;
    BVHBlockData rootBlock = null;
    BVHBlockData parentBlock;
    BVHBlockData currentBlock;
    Stack<BVHBlockData> blockStack = new Stack(); //パース中にparentBlockを保存するために使う
    //上のparentBlock,currentBlock,blockStackあたりの同期は，
    //各ブロックのcaseA???のメソッドに入った瞬間にやることとする．
    //また，今回は混乱しそうなので，inA???,outA???のメソッドは
    //使わないことにする．
    ArrayList<BVHBlockData> blockQueue = new ArrayList(); //パース後に全データを取出すのに使う
    BVHBehavior behaviors[];
    ChannelData channels[];
    int frameCount;
    double frameTime;

    public void caseAHierarchyPart(AHierarchyPart node) {
        node.getRootBlock().apply(this);

        ArrayList bTmp = new ArrayList();
        ArrayList cTmp = new ArrayList();
        for (BVHBlockData d : blockQueue) {
            bTmp.add(d.behavior);
            for (int i=0;i<d.channels.length;i++)
                cTmp.add(d.channels[i]);
        }
        behaviors = (BVHBehavior[])bTmp.toArray(new BVHBehavior[0]);
        channels = (ChannelData[])cTmp.toArray(new ChannelData[0]);
    }

    public void caseARootBlock(ARootBlock node) {
        rootBlock = new BVHBlockData();
        currentBlock = rootBlock;
        blockQueue.add(currentBlock);

        currentBlock.identifier = node.getIdentifier().getText();

        node.getOffsetLine().apply(this);
        node.getChannelsLine().apply(this);

        makeJ3DGroup();

        Object temp[] = node.getSuperBlock().toArray();
        for(int i = 0; i < temp.length; i++) {
            ((PSuperBlock) temp[i]).apply(this);
        }
    }

    public void caseAOffsetLine(AOffsetLine node) {
        PNumber x_number = node.getXOffset();
        PNumber y_number = node.getYOffset();
        PNumber z_number = node.getZOffset();

        currentBlock.offsetX = pNumber2Double(x_number);
        currentBlock.offsetY = pNumber2Double(y_number);
        currentBlock.offsetZ = pNumber2Double(z_number);
    }

    public void caseAChannelsLine(AChannelsLine node) {
        TInteger integer = node.getInteger();
        int channelsCount = Integer.parseInt(integer.getText());

        Object temp[] = node.getChannelType().toArray();
        if (channelsCount != temp.length) {
            System.out.println("ActionGenerator.caseAChannelsBlock()");
            System.out.println("pars error!!!");
        }

        ChannelData cds[] = new ChannelData[temp.length];
        for (int i=0;i<temp.length;i++) {
            ChannelData cd = null;
            if (temp[i] instanceof AXPositionChannelType) {
                cd = new ChannelData("Xposition");
            } else if (temp[i] instanceof AYPositionChannelType) {
                cd = new ChannelData("Yposition");
            } else if (temp[i] instanceof AZPositionChannelType) {
                cd = new ChannelData("Zposition");
            } else if (temp[i] instanceof AXRotationChannelType) {
                cd = new ChannelData("Xrotation");
            } else if (temp[i] instanceof AYRotationChannelType) {
                cd = new ChannelData("Yrotation");
            } else if (temp[i] instanceof AZRotationChannelType) {
                cd = new ChannelData("Zrotation");
            } else if (temp[i] instanceof AXScaleChannelType) {
                cd = new ChannelData("Xscale");
            } else if (temp[i] instanceof AYScaleChannelType) {
                cd = new ChannelData("Yscale");
            } else if (temp[i] instanceof AZScaleChannelType) {
                cd = new ChannelData("Zscale");
            }
            cds[i] = cd;
        }
        currentBlock.channels = cds;
    }

    public void makeJ3DGroup() {
        TransformGroup tg0 = new TransformGroup();
        Transform3D t0 = new Transform3D();
        t0.set(new Vector3d(currentBlock.offsetX,
                            currentBlock.offsetY,
                            currentBlock.offsetZ));
        tg0.setTransform(t0);

        TransformGroup tg1 = new TransformGroup();
        tg1.setCapability(TransformGroup.ALLOW_TRANSFORM_WRITE);
        Transform3D t1 = new Transform3D();
        tg1.setTransform(t1);

        ChannelData cds[] = currentBlock.channels;
        BVHBehavior b = new BVHBehavior(currentBlock.identifier,tg1,t1,cds);
        currentBlock.behavior = b;
        b.setSchedulingBounds(bs);
        tg1.addChild(b);
        tg0.addChild(tg1);

        Point3d[] vertex = new Point3d[2];
        vertex[0] = new Point3d(0.0,0.0,0.0);
        vertex[1] = new Point3d(currentBlock.offsetX,
                                currentBlock.offsetY,
                                currentBlock.offsetZ);
        LineArray geometry = new LineArray(vertex.length,
            GeometryArray.COORDINATES | GeometryArray.COLOR_3);
        geometry.setCoordinates(0,vertex);
        geometry.setColor(0, new Color3f(java.awt.Color.blue));
        geometry.setColor(1, new Color3f(java.awt.Color.red));
        Shape3D shape = new Shape3D(geometry);

        if (rootGroup == null) {
            rootGroup = tg0;
            currentBlock.j3dGroup = tg1;
        } else {
            parentBlock.j3dGroup.addChild(tg0);
            parentBlock.j3dGroup.addChild(shape);
            currentBlock.j3dGroup = tg1;
        }
    }

    public void caseAJointBlock(AJointBlock node) {
        blockStack.push(parentBlock);
        parentBlock = currentBlock;
        currentBlock = new BVHBlockData();
        blockQueue.add(currentBlock);

        currentBlock.identifier = node.getIdentifier().getText();
        node.getOffsetLine().apply(this);
        node.getChannelsLine().apply(this);

        makeJ3DGroup();

        Object temp[] = node.getSuperBlock().toArray();
        for(int i = 0; i < temp.length; i++)
                ((PSuperBlock) temp[i]).apply(this);

        currentBlock = parentBlock;
        parentBlock = blockStack.pop();
    }

    public void caseAEndSiteBlock(AEndSiteBlock node) {
        blockStack.push(parentBlock);
        parentBlock = currentBlock;
        currentBlock = new BVHBlockData();

        currentBlock.identifier = null;
        node.getOffsetLine().apply(this);

        makeJ3DGroup();

        currentBlock = parentBlock;
        parentBlock = blockStack.pop();
    }

    public void caseAMotionPart(AMotionPart node) {
        node.getFramesLine().apply(this);
        node.getFrameTimeLine().apply(this);

        Object temp[] = node.getNumber().toArray();
        if (temp.length != (channels.length*frameCount)) {
            System.out.println("ActionGenerator.caseAMotionPart().");
            System.out.println("parse error!!!");
            System.out.print(temp.length);
            System.out.print(" and "+channels.length*frameCount);
            System.out.println(" do not match.");
            return;
        }

        int tempCount = 0;
        for (int i=0;i<frameCount;i++) {
            for (int j=0;j<channels.length;j++) {
                PNumber pn = (PNumber)temp[tempCount++];
                double d = pNumber2Double(pn);
                channels[j].add(new Double(d));
            }
        }

        for (int i=0;i<behaviors.length;i++) {
            behaviors[i].calc();
        }
    }

    public void caseAFramesLine(AFramesLine node) {
        TInteger integer = node.getInteger();
        String is = integer.getText();
        frameCount = Integer.parseInt(is);

        for (int i=0;i<behaviors.length;i++)
            behaviors[i].setFrameCount(frameCount);
    }

    public void caseAFrameTimeLine(AFrameTimeLine node) {
        PNumber num = node.getNumber();
        frameTime = pNumber2Double(num);
        for (int i=0;i<behaviors.length;i++)
            behaviors[i].setFrameTime(frameTime);
    }

//----------

    protected double pNumber2Double(PNumber num) {
        if (num instanceof ARealNumberNumber) {
            TRealNumber rn = ((ARealNumberNumber)num).getRealNumber();
            String rns = rn.getText();
            return Double.parseDouble(rns);
        } else {
            TInteger integer = ((AIntegerNumber)num).getInteger();
            int iii = Integer.parseInt(integer.getText());
            return (double)iii;
        }
    }
}
